package liv.jmxmonitor;

import java.lang.management.ManagementFactory;
import java.util.concurrent.TimeUnit;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import com.sun.jdmk.comm.HtmlAdaptorServer;

/**
 * Example of using JMX for monitoring certain aspects of a system. Driver
 * program. This is a simple program which spins a separate worker thread that
 * generates 3 random numbers and then sleeps for a random period. The 3
 * generated random numbers are added to a hashmap which maps the number to the
 * number of occurrences and also they are added to a "total".
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class JMXExample {
    /**
     * TCP/IP port to start the HTML/JMX interface on.
     */
    public static final int  JMX_HTTP_PORT    = 8000;

    /**
     * How many numbers to generate at each go
     */
    public static final int  GENERATE_NUMBERS = 3;

    /**
     * Maximum sleep time in between generating numbers
     */
    public static final long SLEEP_TIME       = 1000;

    /**
     * Maximum value of the random number generated.
     */
    public static final int  MAX_NUMBER       = 100;

    /**
     * Frequency map to use.
     */
    private FreqMap          map;

    public JMXExample() {
        map = new FreqMap();
    }

    /**
     * Program entry point.
     *
     * @param args
     *            command line parameters -- ignored.
     */
    public static void main(String[] args) {
        JMXExample driver = new JMXExample();
        try {
            driver.start();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * Used to register {@link #map} with JMX as an MBean and start the Sun
     * HTML/JMX interface.
     */
    public void prepareJMX() throws Exception {
        /*
         * Register stats
         */
        MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
        ObjectName mapName = new ObjectName("STATS:name=FrequencyMap");
        mbs.registerMBean(this.map, mapName);

        /*
         * This is the crucial part -- starts the JMX / HTTP interface
         */
        /* * */
        HtmlAdaptorServer adaptor = new HtmlAdaptorServer();
        adaptor.setPort(JMX_HTTP_PORT);
        ObjectName adapterName = new ObjectName("HTMLAgent:name=htmladapter,port=" + JMX_HTTP_PORT);
        mbs.registerMBean(adaptor, adapterName);
        adaptor.start();
        /* * */
    }

    public void start() throws Exception {
        prepareJMX();

        int rnd[] = new int[GENERATE_NUMBERS];
        while (true) {
            // do this forever
            for (int i = 0; i < rnd.length; i++)
                rnd[i] = (int) (Math.random() * MAX_NUMBER);
            map.add(rnd);
            long sleep = (long) (Math.random() * SLEEP_TIME);
            TimeUnit.MILLISECONDS.sleep(sleep);
        }
    }
}
